// Flexradioconnect
// Copyright 2016,2017 Hardy Lau, DL1GLH
//
// v171213-0.93 Alpha
//
// Standalone interface for interconnection of
// + Flexradio 6300/6500/6700 
// + S.P.E. Expert 2K-FA
// + Ultrabeam dynamic antenna
//
// Running on Banana Pro
// This part does connect to a VSNETCOM Serial-to-Network Sever Port
// to control a Expert 2K-FA Amplifier
//

/*
 Copyright 2016,2017 Hardy Lau, DL1GLH

 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

 3. Neither the name of the copyright holder nor the names of its contributors
    may be used to endorse or promote products derived from this software without
    specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <stdio.h>
#include <stdlib.h>
#include <netdb.h>
#include <netinet/in.h>
#define _GNU_SOURCE
#include <string.h>
#include <sys/types.h>
#include <dirent.h>
#include <signal.h>
#include <unistd.h>
#include <time.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#define DEBUG
#undef DEBUG

// VSNETCOM Network-to-Serial Server with network accessable port
// Expert 2K-FA Commands
#define SERIALTONETSERVER "spe2kfa-034.inf.dhbw-ravensburg.de"
#define DATAPORT 2001

//Main
//
int main( int argc, char *argv[])
{
   int sockfd, portno, n;
   FILE *sock;
   struct sockaddr_in serv_addr;
   struct hostent *server;
   fd_set rfds;
   struct timeval tv;
   int retval;
   int i;
   char buffer[200];
   struct itimerval timer;
   
   int command = 0;

   /* Configure the timer to expire after 800 msec... the alarm will kill the process */
   timer.it_value.tv_sec = 0;
   timer.it_value.tv_usec = 800000;
   timer.it_interval.tv_sec = 0;
   timer.it_interval.tv_usec = 0;
   setitimer (ITIMER_REAL, &timer, NULL);

   printf( "Content-type: text/html\n\n");
   fflush(stdout);
   printf( "<html><head></head><body>\n");
   
   if( getenv("REQUEST_METHOD") != NULL && !strcmp( "GET", getenv("REQUEST_METHOD"))) {
     if( getenv("QUERY_STRING") != NULL) {
       strcpy( buffer, getenv("QUERY_STRING"));
       if( strstr( buffer, "cmd=")) {
         sscanf( buffer, "cmd=%d", &command); 
       }
     }
   }
 
   /* Create a socket point */
   sockfd = socket(AF_INET, SOCK_STREAM, 0);
   
   if (sockfd < 0) {
      perror("ERROR opening socket");
      exit(1);
   }
	
   server = gethostbyname(SERIALTONETSERVER);
   
   if (server == NULL) {
      fprintf(stderr,"No such host name - trying ip number...\n");
      server = gethostbyaddr((char *)inet_addr(SERIALTONETSERVER), 4, AF_INET);
      if(server == NULL) {
        perror("ERROR - No such host...\n");
        exit(1);
      }
   }
   
   bzero((char *) &serv_addr, sizeof(serv_addr));
   serv_addr.sin_family = AF_INET;
   bcopy((char *)server->h_addr, (char *)&serv_addr.sin_addr.s_addr, server->h_length);
   serv_addr.sin_port = htons(DATAPORT);
   
   /* Now connect to the serial-to-network server */
   if (connect(sockfd, (struct sockaddr*)&serv_addr, sizeof(serv_addr)) < 0) {
      perror("ERROR connecting");
      exit(1);
   }

   /* Control if requested
   */
   if( command > 0) {
     switch( command) {
       case 1: write(sockfd, "\125\125\125\001\015\015", 6);  //OPEATE 0x0D
               break;
       case 2: write(sockfd, "\125\125\125\001\001\001", 6);  //INPUT 0x01
               break;
       case 3: write(sockfd, "\125\125\125\001\013\013", 6);  //POWER 0x0B
               break;
     }
     if( read( sockfd, buffer, 6) != 6) {
       printf( "ERROR Controlling Expert Amplifier");
       exit(1);
     }
   }

   /* Read Status 
   */
   
   if( write(sockfd, "\125\125\125\001\220\220", 6) <= 0) {
     perror("ERROR writing");
     exit(1);
   }

   n = 0;
   do {
     if( read( sockfd, buffer + n, 1) < 1) {
       perror("ERROR reading");
       exit(1);
     }
     n++;
     buffer[n] = '\0';
   } while( buffer[n-1] != '\n' && n < 75);

   if( n == 75 && buffer[0] == 0xaa && buffer[3] == 67) {
     if( !strncmp( buffer + 5, "20K", 3)) printf( "2K-FA");
     if( !strncmp( buffer + 5, "13K", 3)) printf( "1.3K-FA");
     if( !strncmp( buffer + 5, "15K", 3)) printf( "1.5K-FA");
     printf( "&nbsp;&nbsp;");
     if( buffer[9] == 'S') printf( "STANDBY");
     if( buffer[9] == 'O') printf( "OPER");
     printf( "&nbsp;&nbsp;");
     if( buffer[11] == 'R') printf( "RX");
     if( buffer[11] == 'T') printf( "TX");
     printf( "&nbsp;&nbsp;");
     if( buffer[15] == '1') printf( "INPUT:1");
     if( buffer[15] == '2') printf( "INPUT:2");
     printf( "&nbsp;&nbsp;");
     if( !strncmp( buffer + 17, "00", 2)) printf( "160m");
     if( !strncmp( buffer + 17, "01", 2)) printf( "80m");
     if( !strncmp( buffer + 17, "02", 2)) printf( "60m");
     if( !strncmp( buffer + 17, "03", 2)) printf( "40m");
     if( !strncmp( buffer + 17, "04", 2)) printf( "30m");
     if( !strncmp( buffer + 17, "05", 2)) printf( "20m");
     if( !strncmp( buffer + 17, "06", 2)) printf( "17m");
     if( !strncmp( buffer + 17, "07", 2)) printf( "15m");
     if( !strncmp( buffer + 17, "08", 2)) printf( "12m");
     if( !strncmp( buffer + 17, "09", 2)) printf( "10m");
     if( !strncmp( buffer + 17, "10", 2)) printf( "6m");
     if( !strncmp( buffer + 17, "11", 2)) printf( "4m");
     printf( "&nbsp;&nbsp;");
     printf( "ANT:%c%c", buffer[20], buffer[21]);
     printf( "&nbsp;&nbsp;");
     if( buffer[26] == 'L') printf( "POWER:LOW");
     if( buffer[26] == 'M') printf( "POWER:MID");
     if( buffer[26] == 'H') printf( "POWER:MAX");
     printf( "&nbsp;&nbsp;");
     printf( "Temp:%c%c%c", buffer[55],buffer[56],buffer[57]);
     printf( "&nbsp;&nbsp;<br>");
     if( buffer[11] == 'T') { //TX
       printf( "Output-Power: %c%c%c%c", buffer[28],buffer[29],buffer[30],buffer[31]);
       printf( "&nbsp;&nbsp;");
       printf( "SWR-ATU: %c%c%c%c%c", buffer[33],buffer[34],buffer[35],buffer[36],buffer[37]);
       printf( "&nbsp;&nbsp;");
       printf( "SWR-ANT: %c%c%c%c%c", buffer[39],buffer[40],buffer[41],buffer[42],buffer[43]);
       printf( "&nbsp;&nbsp;");
       printf( "V-PA: %c%c%c%c", buffer[45],buffer[46],buffer[47],buffer[48]);
       printf( "&nbsp;&nbsp;");
       printf( "I-PA: %c%c%c%c", buffer[50],buffer[51],buffer[52],buffer[53]);
       printf( "&nbsp;&nbsp;");
     }
     if( buffer[67] != 'N') {
       printf( "<br>WARNING: ");
       switch( buffer[67]) {
         case 'M': printf( "ALARM AMPLIFIER"); 
                   break;
         case 'A': printf( "NO SELECTED ANTENNA");
                   break;
         case 'S': printf( "SWR ANTENNA");
                   break;
         case 'B': printf( "NO VALID BAND");
                   break;
         case 'P': printf( "POWER LIMIT EXCEEDED");
                   break;
         case 'O': printf( "OVERHEATING");
                   break;
         case 'Y': printf( "ATU NOT AVAILABLE");
                   break;
         case 'W': printf( "TUNING WITH NO POWER");
                   break;
         case 'K': printf( "ATU BYPASSED");
                   break;
         case 'R': printf( "POWER SWITCH HELD BY REMOTE");
                   break;
         case 'T': printf( "COMBINER OVERHEATING");
                   break;
         case 'C': printf( "COMBINER FAULT");
                   break;
       }  
       printf( "&nbsp;&nbsp;");
     }
     if( buffer[69] != 'N') {
       printf( "<br>ERROR: ");
       switch( buffer[69]) {
         case 'S': printf( "SWR EXCEEDING LIMITS");
                   break;
         case 'A': printf( "AMPLIFIER PROTECTION");
                   break;
         case 'D': printf( "INPUT OVERDRIVING");
                   break;
         case 'H': printf( "EXCESS OVERHEATING");
                   break;
         case 'C': printf( "COMBINER FAULT");
                   break;
       }
     }
   } else printf( "Status %d Bytes read.", n);

   close( sockfd);

   printf( "</body></html>");
}
