// Flexconnect
// Cpyright 2016,2017 Hardy Lau, DL1GLH
//
// v171213-0.93 Alpha
//
// Standalone interface for interconnection of
// + Flexradio Series 6x00
// + S.P.E. Expert 2K-FA
// + Ultrabeam dynamic antenna
//
// Running on Banana Pro
// This part does show status on webserver (cgi)
//

/*
 Copyright 2016,2017 Hardy Lau, DL1GLH

 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

 3. Neither the name of the copyright holder nor the names of its contributors
    may be used to endorse or promote products derived from this software without
    specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "connect.h"
#include <stdio.h>
#include <stdlib.h>
#include <netdb.h>
#include <netinet/in.h>
#define _GNU_SOURCE
#include <string.h>
#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h>
#include <sys/types.h>

//read a file /run/flexconnect/frequency-%1.6lf
//
double getfrequency()
{
 DIR *dir;
 struct dirent *dp;
 char *file_name;
 char file_name_full_path[200];
 int fd;
 double frequency = 0.0;
 int x = 30;
 int found = 0;

 dir = opendir( "/run/flexconnect");
 if( dir < 0) return( -1.0);

 while ((dp=readdir(dir)) != NULL && x--) {
   file_name = dp->d_name;
   if( strstr( file_name, "frequency-")) {
     sscanf( file_name, "frequency-%lf", &frequency);
     found++;
   }
 }
 closedir( dir);
 return( found ? frequency : -1.0);
}

//Read UB status file  /run/flexconnect/ub-status-*
//
char* read_ub_status()
{
 DIR *dir;
 struct dirent *dp;
 char *file_name;
 char file_name_full_path[200];
 static char returnstring[200];
 char tempstring[200];
 unsigned int low, high;

 strcpy( returnstring, "");

 //Get status files
 dir = opendir( "/run/flexconnect");
 while ((dp=readdir(dir)) != NULL) {
   file_name = dp->d_name;
   if( strstr( file_name, "ub-status-")) {
     if( strncmp( file_name + 45, "-00-", 4))
       strcat( returnstring, "ELEMENTS MOVING - ");
     if( !strncmp( file_name + 36, "-00-", 4))
       strcat( returnstring, "NORMAL -");
     if( !strncmp( file_name + 36, "-01-", 4))
       strcat( returnstring, "REVERSE 180&deg; -");
     if( !strncmp( file_name + 36, "-02-", 4))
       strcat( returnstring, "BIMODE -");
     sscanf( file_name + 28, "%02x-%02x", &low, &high);
     sprintf( tempstring, " %d Kc", low + 256 * high);
     strcat( returnstring, tempstring);
 
//     strcat( returnstring, basename(file_name));
   }
 }
 closedir( dir);
 return( returnstring);
}

int main()
{
 printf( "Content-type: text/html\n\n");
 fflush(stdout);
 //printf( "<html><head></head><body>\n");
 if( getfrequency() > 0.0)
   printf( "Frequency: %.3lf Kc", getfrequency() * 1000);
 else if( getfrequency() == 0.0)
   printf( "Flexradio in ON but has no active slice");
 else
   printf( "Flexradio is OFF");
 printf( "<br>");
 printf( "UltraBeam-Status: %s", read_ub_status());
 //printf( "</body></html>");
}
